//Функция генерации обучающих данных (для динамики выходных характеристик)
function [SystemOutputCharacteristics,..//Выходные характеристики
          SystemParametrs..//Параметры системы, для которых рассчитывались выходные характеристики
          ]=AdvancedModelLearningDataTimeGrid(SystemOutputCharacteristicsNames,...//Имена выходных характеристик системы
                                              SystemOutputCharacteristicsNamesIndexes,...//Индексы выходных характеристик системы
                                              SystemOutputCharacteristicsIndexes,...//Индексы выходных характеристик системы (в именах)
                                              SystemShemaName,...//Имя схемы системы
                                              TimeMoments,...//Моменты времени
                                              SystemParametrsNames,...//Имена параметров, определяющих динамику
                                              SystemParametrsValues,...//Значения параметров, определяющих динамику
                                              absolutTolErr,..//Максимальное значение абсолютной ошибки
                                              relativeTolErr,..//Минимальное значение относительной ошибки
                                              simulationTime,..//Время симуляции
                                              timeStepMax,..//Максимальный шаг интергирования по времени
                                              Solver,..//Решатель
                                              toleranceTime..//Максимальная ошибка времени
                                              )
    //Проверяем число аргументов
    nArg = argn(2);//Число аргументов
    if (nArg < 5) || (nArg == 6) then
        error('Function AdvancedModelLearningDataTimeGrid takes 5, 7 or more arguments');
    end
    
    //Проверяем корректность аргументов
    sizeSystemOutputCharacteristicsNames = size(SystemOutputCharacteristicsNames);
    if type(SystemOutputCharacteristicsNames) ~= 10 then
        error('The first argument of function AdvancedModelLearningDataTimeGrid is matrix of string');
    end
    if sizeSystemOutputCharacteristicsNames(1) ~= 1 then
        error('The first argument of function AdvancedModelLearningDataTimeGrid is matrix 1xM')
    end
    sizeSystemOutputCharacteristicsNamesIndexes = size(SystemOutputCharacteristicsNamesIndexes);
    if type(SystemOutputCharacteristicsNamesIndexes) ~= 1 then
        error('The second argument of function AdvancedModelLearningDataTimeGrid is matrix of integer');
    end
    sizeSystemOutputCharacteristicsIndexes = size(SystemOutputCharacteristicsIndexes);
    if type(SystemOutputCharacteristicsIndexes) ~= 1 then
        error('The third argument of function AdvancedModelLearningDataTimeGrid is matrix of integer');
    end
    if (sizeSystemOutputCharacteristicsNames ~= sizeSystemOutputCharacteristicsNamesIndexes) ||..
       (sizeSystemOutputCharacteristicsIndexes ~= sizeSystemOutputCharacteristicsNamesIndexes) then
        error('The first, second and third arguments must be matrix same dimensions 1xM');
    end
    if (type(SystemShemaName) ~= 10) ||..
       (size(SystemShemaName, '*') > 1) then
        error('The 4th argument of function AdvancedModelLearningDataTimeGrid is scalar string');
    end
    if type(TimeMoments) ~= 17 then
        error('The 5th argument of function AdvancedModelLearningDataTimeGrid is cell array of real vectors Mx1');
    end
    if (nArg > 6) then
        sizeSystemParametrsNames = size(SystemParametrsNames);
        if ~isempty(SystemParametrsNames) &&..
           (type(SystemParametrsNames) ~= 10) then
            error('The 6th argument of function AdvancedModelLearningDataTimeGrid is matrix of strings');
        end
        if sizeSystemParametrsNames(1) > 1 then
            error('The 6th argument of function AdvancedModelLearningDataTimeGrid is matrix 1xM');
        end
        sizeSystemParametrsValues = size(SystemParametrsValues);
        if type(SystemParametrsValues) ~= 1 then
            error('The 7th argument of function AdvancedModelLearningDataTimeGrid is matrix of real');
        end
        if sizeSystemParametrsNames ~= sizeSystemParametrsValues then
            error('The 7th argument of function AdvancedModelLearningDataTimeGrid is matrix 1xM');
        end
    end
    if nArg > 7 then
        IsCorrectNumericIntegrationAttributesTimeGrid(absolutTolErr,..
                                                      sizeSystemParametrsNames);
    end
    if nArg > 8 then
        IsCorrectNumericIntegrationAttributesTimeGrid(relativeTolErr,..
                                                      sizeSystemParametrsNames);
    end
    if nArg > 9 then
        IsCorrectNumericIntegrationAttributesTimeGrid(simulationTime,..
                                                      sizeSystemParametrsNames);
    end
    if nArg > 10 then
        IsCorrectNumericIntegrationAttributesTimeGrid(timeStepMax,..
                                                      sizeSystemParametrsNames);
    end
    if nArg > 11 then
        IsCorrectNumericIntegrationAttributesTimeGrid(Solver,..
                                                      sizeSystemParametrsNames); 
    end
    if nArg > 12 then
        IsCorrectNumericIntegrationAttributesTimeGrid(toleranceTime,..
                                                      sizeSystemParametrsNames); 
    end
                                             
    //Восполняем недостающие аргументы
    if nArg < 6 then
        SystemParametrsNames = [];
        SystemParametrsValues = [];
    end
    if nArg < 8 then
        absolutTolErr = [];
    end
    if nArg < 9 then
        relativeTolErr = [];
    end
    if nArg < 10 then
        simulationTime = [];
    end
    if nArg < 11 then
        timeStepMax = [];
    end
    if nArg < 12 then
        Solver = [];
    end
    if nArg < 13 then
        toleranceTime = [];
    end
    
    //Загружаем схему системы
    SystemShema = xcosDiagramToScilab(SystemShemaName);
    
    //Векторизуемая функция вычислительного эксперимента
    nCurves = size(SystemParametrsValues)(1);//Число кривых
    SystemParametrs = [];
    SystemOutputCharacteristics = [];
    for ind=1:nCurves
        //Проводим вычислительный эксперимент
        systemOutputCharacteristics = ComputingExperiment(SystemOutputCharacteristicsNames,...//Имена выходных характеристик системы
                                                          SystemShema,..//Схема системы
                                                          SystemParametrsNames,..//Имена параметров
                                                          SystemParametrsValues(ind,:),..//Значения параметров
                                                          absolutTolErr(ind),..//Максимальное значение абсолютной ошибки
                                                          relativeTolErr(ind),..//Минимальное значение относительной ошибки
                                                          simulationTime(ind),..//Время симуляции
                                                          timeStepMax(ind),..//Максимальный шаг интергирования по времени
                                                          Solver(ind),..//Решатель
                                                          toleranceTime(ind)..//Максимальная ошибка времени
                                                          );
                                                          
        //Приводим выходные характеристики системы к одному временному масштабу
        DynamicsInTimeGrid = DynamicsToTimeGrid(systemOutputCharacteristics,..//Массив динамик
                                                SystemOutputCharacteristicsNamesIndexes,..//Индексы имен динамик
                                                SystemOutputCharacteristicsIndexes,..//Индексы динамик (в именах)
                                                TimeMoments{ind}..//Моменты времени
                                                );
                                                              
        //Приводим параметры и формируем результат
        SystemParametrs = [SystemParametrs;..
                           repmat(SystemParametrsValues(ind,:)',..
                                  [1,size(DynamicsInTimeGrid)(1)])'];
        SystemOutputCharacteristics = [SystemOutputCharacteristics;..
                                       DynamicsInTimeGrid];
    end
endfunction

//Приведение динамик к единой заданной пользователем временной сетке
function [DynamicsInTimeGrid...//Динамики в единой временной сетке
          ]=DynamicsToTimeGrid(Dynamics,...//Массив динамик
                               DynamicsNamesIndexes,...//Индексы имен динамик
                               DynamicsIndexes,...//Индексы динамик (в именах)
                               TimesMoments...//Моменты времени
                               )
    //Проверяем корректность аргументов
    if isempty(Dynamics) || isempty(DynamicsNamesIndexes) ||..
       isempty(DynamicsIndexes) || isempty(TimesMoments)  then
        DynamicsInTimeGrid = [];
        return;
    end
    
    //Выполняем фильтрацию
    DynamicsInTimeGrid = [];
    for ind=1:size(DynamicsIndexes,'*')//Вспомогательная функция фильтрации по индексу
        dynamicAfterConvercion = interp1(Dynamics(DynamicsNamesIndexes(ind)).time,...
                                         Dynamics(DynamicsNamesIndexes(ind)).values(:,DynamicsIndexes(ind)),...
                                         TimesMoments, "spline");
        DynamicsInTimeGrid = [DynamicsInTimeGrid,..
                              dynamicAfterConvercion];
    end
endfunction

//Проверяем аргументы численных процессов интегрирования
function IsCorrectNumericIntegrationAttributesTimeGrid(attr,sizeEtalon)
    sizeAttr = size(attr);
    if sizeAttr(1) > 1 then
        error('The 8th argument of function AdvancedModelLearningDataTimeGrid is matrix 1xM');
    end
    if type(attr) ~= 1 then
        error('The 8th argument of function AdvancedModelLearningDataTimeGrida is matrix if real');
    end
    if (sizeAttr(2) > 0) &&..
       (sizeAttr ~= sizeEtalon) then
        error('The 6th and 8th argument of function AdvancedModelLearningDataTimeGrid are metrixes same dimensions 1xM');
    end
endfunction
