//Моделируем динамику
function [SystemOutputChatacteristics,..//Выходные характеристики системы
          SystemProps..//Свойства системы
          ]=ComputingExperiment(SystemOutputChatacteristicsNames,...//Имена выходных характеристик системы
                                SystemShema,...//Схема системы
                                SystemParametrsNames,...//Имена параметров
                                SystemParametrsValues,...//Значения параметров
                                absolutTolErr,..//Максимальное значение абсолютной ошибки
                                relativeTolErr,..//Минимальное значение относительной ошибки
                                simulationTime,..//Время симуляции
                                timeStepMax,..//Максимальный шаг интергирования по времени
                                Solver,..//Решатель
                                toleranceTime..//Максимальная ошибка времени
                                )//Моделирование возможных динамик
    //Проверяем число аргументов
    nArgs = argn(2);
    if (nArgs ~= 2) && (nArgs < 4) then
        error('Function ComputingExperiment takes 2, 4 or more arguments (maximum 10 arguments)');
    end
    
    //Проверяем корректность типов данных аргументов
    if type(SystemOutputChatacteristicsNames) ~= 10 then
        error('The first argument of ComputingExperiment function is array of string');
    end
    if type(SystemShema) ~= 128 then
        error('The second argument of ComputingExperiment function is array of system sheme diagram type exemplar');
    end
    if nArgs >= 4 then
        if ~isempty(SystemParametrsNames) && (type(SystemParametrsNames) ~= 10) then
            error('The third argument of ComputingExperiment function is array of string');
        end
        if type(SystemParametrsValues) ~= 1 then
            error('The 4th argument of ComputingExperiment function is array of real');
        end
        sizeSystemParametrsNames = size(SystemParametrsNames);
        if or(sizeSystemParametrsNames ~= size(SystemParametrsValues)) ||...
           (sizeSystemParametrsNames(1) > 1) then
            error('The third and 4th argument of ComputingExperiment function are matrixes same dimencions 1xM');
        end
    end
    TryScalarRealType(absolutTolErr,5);
    TryScalarRealType(relativeTolErr,6);
    TryScalarRealType(simulationTime,7);
    TryScalarRealType(timeStepMax,8);
    TryScalarRealType(Solver,9);
    TryScalarRealType(toleranceTime,10);
    
    //задаем параметры и контекст моделирования
    if nArgs >= 4 then
        SystemShema.props.context = SetContext(SystemParametrsNames,...//Имена параметров
                                               SystemParametrsValues,...//Значения параметров
                                               SystemShema.props.context...//Задаваемый контекст
                                               );//Задаем параметры системы
    end
    if (nArgs >= 5) && ~isempty(absolutTolErr) then
        SystemShema.props.tol(1) = absolutTolErr;//Задаем абсолютную погрешность
    end
    if (nArgs >= 6) && ~isempty(relativeTolErr) then
        SystemShema.props.tol(2) = relativeTolErr;//Задаем относительную погрешность
    end
    if (nArgs >= 7) && ~isempty(simulationTime) then
        SystemShema.props.tf = simulationTime;//Задаем время симуляции
    end
    if (nArgs >= 8) && ~isempty(timeStepMax) then
        SystemShema.props.tol(7) = timeStepMax;//Задаем время симуляции
    end
    if (nArgs >= 9) && ~isempty(Solver) then
        SystemShema.props.tol(6) = Solver;//Задаем решатель
    end
    if (nArgs >= 10) && ~isempty(toleranceTime) then
        SystemShema.props.tol(3) = toleranceTime;//Задаем погрешность для времени
    end
    SystemShema.props.tol(5) = 0.0;//Отключаем реальное время
                                      
    //Запускаем моделирование динамики
    xcos_simulate(SystemShema, 4);
    
    //Получаем значения выходных характеристик системы
    SystemOutputChatacteristics = evstr(SystemOutputChatacteristicsNames);
    
    //Возвращаем свойства системы
    SystemProps = SystemShema.props;
endfunction

//Вспомогательные функции
function [context]=SetContext(ParametrsNames,...//Имена параметров
                              ParametrsValues,...//Значения параметров
                              context...//Задаваемый контекст
                              )//задаем контекст
    //Проверяем аргументы
    if isempty(ParametrsNames) || isempty(ParametrsValues) ||..
       isempty(context) then
        return;
    end
    
    //Выделяем из контекста имена величин
    contextNames = stripblanks(strsplit(strcat(context, ' = '), '=')(1:2:2*size(context, '*')));
    
    //Приравниваем именам величин их значения
    parametrs = ParametrsNames + '=' + string(ParametrsValues);
    
    //Определяем индексы заданных величин в контексте
    parametrsNames = stripblanks(ParametrsNames);
    indexes = FindParametrInHeader(parametrsNames, contextNames);
    
    //Заполняем контекст
    context(indexes) = parametrs;
endfunction
function TryScalarRealType(val,nArgs)//Проверяем реальный скалярный тип данных
    if (nArgs >= 6) && ((size(val,'*') > 1) ||..
       (type(val) ~= 1)) then
       error('The ' + nArgs + 'th argument of ComputingExperiment function are real scalar');
    end
endfunction
function [indexes,existing]=FindParametrInHeader(Array, ArrayBase)//Получение индексов
    //Проверяем число аргументов
    if argn(2) ~= 2 then
        error('Function Find takes 2 arguments');
        return;
    end
    
    //Выполняем поиск
    [existing, indexes] = members(Array, ArrayBase);
    existing = (existing > 0); indexes = indexes(existing);
endfunction
